/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <errno.h>

#include "common.h"
#include "log.h"
#include "action.h"

int faultd_fill_for_srv_restart(struct faultd_object *obj, char *service_path)
{
	int ret;

	ret = faultd_object_append_string(obj, FAULTD_AD_SERVICE_NAME, service_path);
	if (ret < 0) {
		log_error("Unable to append %s:%s",
				  FAULTD_AD_SERVICE_NAME, service_path);
		return -ENOMEM;
	}

	return 0;
}

int faultd_fill_for_srv_recover(struct faultd_object *obj, char *service_path, char *recovery)
{
	int ret;

	ret = faultd_object_append_string(obj, FAULTD_AD_SERVICE_NAME, service_path);
	if (ret < 0) {
		log_error("Unable to append %s:%s",
				  FAULTD_AD_SERVICE_NAME, service_path);
		return -ENOMEM;
	}

	if (recovery) {
		ret = faultd_object_append_string(obj, FAULTD_AD_CLEANUP_UNIT, recovery);
		if (ret < 0) {
			log_error("Unable to append %s:%s",
					  FAULTD_AD_CLEANUP_UNIT, recovery);
			return -ENOMEM;
		}
	}

	return 0;
}
