/*
 * This file is a part of fauld.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <assert.h>
#include <ejdb/ejdb.h>
#include <errno.h>
#include <systemd/sd-id128.h>

#include "database.h"
#include "log.h"
#include "module.h"

static struct faultd_database_adapter *database_adapter = NULL;

void faultd_set_database_adapter(struct faultd_database_adapter *adapter)
{
	database_adapter = adapter;
}

int database_store(struct faultd_object *obj, faultd_oid_t *oid)
{
	if (!database_adapter)
		return -ENOSYS;

	return database_adapter->store(database_adapter, obj, oid);
}

int database_load(struct faultd_object *query, struct faultd_object *result, uint32_t *nr)
{
	if (!database_adapter)
		return -ENOSYS;

	return database_adapter->load(database_adapter, query, result, nr);
}

int database_new_oid(faultd_oid_t *oid)
{
	if (!database_adapter)
		return -ENOSYS;

	return database_adapter->new_oid(oid);
}

int database_get_by_oid(faultd_oid_t *oid, struct faultd_object *result)
{
	if (!database_adapter)
		return -ENOSYS;

	return database_adapter->get_by_oid(database_adapter, oid, result);
}
