#include <errno.h>
#include <systemd/sd-id128.h>

#include "event.h"
#include "module.h"
#include "common.h"
#include "faultd_started_event.h"

static int fs_event_allocate(struct faultd_event_type *type,
                             void *data, struct faultd_event **ev)
{
	struct faultd_started_event *fs_ev;
	int ret;

	fs_ev = malloc(sizeof(*fs_ev));
	if (!fs_ev)
		return -ENOMEM;

	ret = faultd_event_init_internal(type, &fs_ev->event);
	if (ret)
		goto free_fs_ev;

	*ev = &fs_ev->event;
	return 0;

free_fs_ev:
	free(fs_ev);
	return ret;
}

static void fs_event_release(struct faultd_event *ev)
{
	struct faultd_started_event *fs_ev = 
		to_faultd_started_event(ev);

	faultd_event_cleanup_internal(&fs_ev->event);
	free(fs_ev);
}

static char *fs_event_to_string(struct faultd_event *ev)
{
	char *str;
	int ret;

	ret = asprintf(&str, "Faultd started event"
				   " Time: %lld.%.9ld",
				   (long long)ev->timestamp.tv_sec,
				   ev->timestamp.tv_nsec);

	return ret > 0 ? str : NULL;
}

static void fs_event_serialize(struct faultd_event *ev, struct faultd_object *out)
{
	faultd_event_serialize_internal(ev, out);
}

static struct faultd_event_type faultd_started_event_type = {
	.name = FAULTD_STARTED_EVENT_ID,
	.default_ops = {
		.release = fs_event_release,
		.serialize = fs_event_serialize,
		.to_string = fs_event_to_string,
	},
	.allocate_event = fs_event_allocate,
	.node = LIST_HEAD_INIT(faultd_started_event_type.node),
};

FAULTD_EVENT_TYPE_REGISTER(faultd_started_event_type, faultd_started_et)
