#include "decision_made_event.h"
#include "helpers.h"

#define TESTED_MOD decision_made_event

DECLARE_TEST_ENTRY_POINT(TESTED_MOD, dm_event_serialize, void);

#define KEY1 "somekey"
#define VAL1 "value"

#define KEY2 "somekey2"
#define VAL2 "value2"

static struct faultd_event *alloc_event()
{
	struct decision_made_event *dm_ev;

	dm_ev = calloc(1, sizeof(*dm_ev));
	assert_non_null(dm_ev);

	fill_event(&dm_ev->event);

	dm_ev->reason = calloc(1, sizeof(*(dm_ev->reason)));
	assert_non_null(dm_ev->reason);

	dm_ev->reason->oid = generate_oid();
	dm_ev->who_made = generate_string(STR_SIZE);
	dm_ev->action = generate_string(STR_SIZE);

	faultd_object_new(&dm_ev->action_data);
	faultd_object_append_string(dm_ev->action_data, KEY1, VAL1);
	faultd_object_append_string(dm_ev->action_data, KEY2, VAL2);

	return &dm_ev->event;
}

static void generate_test_attrs(struct serialize_test_data *td)
{
	struct decision_made_event *dm_ev = to_decision_made_event(td->ev);
	struct serialized_element action_data[] = {
		S_ELEMENT(KEY1, TYPE_STRING, VAL1),
		S_ELEMENT(KEY2, TYPE_STRING, VAL2)
	};

	struct serialized_element serialized_attrs[] = {
		S_ELEMENT_EVENT(dm_ev->event),
		S_ELEMENT(DM_EV_REASON, TYPE_OID, &dm_ev->reason->oid),
		S_ELEMENT(DM_EV_WHO, TYPE_STRING, dm_ev->who_made),
		S_ELEMENT(DM_EV_ACTION, TYPE_STRING, dm_ev->action),
		S_ELEMENT_OBJ(DM_EV_ACTION_DATA, dm_ev->action_data,
					  action_data, ARRAY_SIZE(action_data))
	};

	td->test_data = serialized_elems_dup(serialized_attrs,
										 ARRAY_SIZE(serialized_attrs));
	td->size = ARRAY_SIZE(serialized_attrs);
}

static int setup(void **state)
{
	return setup_event_serialization(state, alloc_event, generate_test_attrs,
									 (serialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, dm_event_serialize));
}

static int teardown(void **state)
{
	struct serialize_test_data *td = *state;
	struct decision_made_event *dm_ev = to_decision_made_event(td->ev);

	faultd_object_unref(dm_ev->action_data);
	free_event(&dm_ev->event);
	free(dm_ev->who_made);
	free(dm_ev->action);
	free(dm_ev->reason);
	free(dm_ev);
	serialized_elems_free(td->test_data, td->size);
	free(td);

	return 0;
}

FAULTD_TEST_GROUP(
	FAULTD_TEST_CASE_NST("dm_ev_serialize_test", test_event_serialization,
						 setup, teardown)
)
