#ifndef FAULTD_UNIT_H
#define FAULTD_UNIT_H

#include <stddef.h>
#include <setjmp.h>
#include <stdarg.h>
#include <cmocka.h>

#include "list.h"
#include "common.h"
#include "module.h"

#define CALL_TEST_ENTRY_POINT(MOD, FUNC)		\
	TEST_ENTRY_POINT_NAME(MOD, FUNC)

#define DECLARE_TEST_ENTRY_POINT(MOD, FUNC, RET, ...)		\
	extern RET (*TEST_ENTRY_POINT_NAME(MOD, FUNC))(__VA_ARGS__)

struct fdtf_test_group {
	char *name;
	CMFixtureFunction setup;
	CMFixtureFunction teardown;
	struct CMUnitTest *tests;
	size_t number_of_tests;
	struct GroupTest gt;
	struct list_head node;
};

#ifndef __CONSTRUCTOR__
#define __CONSTRUCTOR__ __attribute__((constructor))
#endif

void fdtf_register_test_group(struct fdtf_test_group *tg);

#define FAULTD_TEST_GROUP(...)								\
	static struct CMUnitTest tests[] = {					\
		__VA_ARGS__											\
	};														\
	static struct fdtf_test_group test_group = {			\
		.name = FAULTD_MODNAME,								\
		.setup = NULL,										\
		.teardown = NULL,									\
		.tests = tests,										\
		.number_of_tests = ARRAY_SIZE(tests),				\
		.node = LIST_HEAD_INIT(test_group.node),			\
	};														\
	static void __CONSTRUCTOR__ register_test_group(void)	\
	{														\
		fdtf_register_test_group(&test_group);				\
	}

#define FAULTD_TEST_CASE_NST(NAME, FUNC, SETUP, TEARDOWN)	\
	{														\
		.name = NAME,										\
		.test_func = FUNC,									\
		.setup_func = SETUP,								\
		.teardown_func = TEARDOWN,								\
	}

#define FAULTD_TEST_CASE_ST(func, setup, teardown)	\
	FAULTD_TEST_CASE_NST(#func, func, setup, teardown)

#define FAULTD_TEST_CASE(func)					\
	FAULTD_TEST_CASE_ST(func, NULL, NULL)

#endif /* FAULTD_UNIT_H */
